import logging as _logging
from simbak import fileutil as _fileutil

_logger = _logging.getLogger(__name__)


class NormalAgent():
    def __init__(self, sources: list, destinations: list, name: str,
                 compression_level: int = 6):
        """Initializer of the NormalAgent object

        Args:
            sources (list of str): Paths to the files that you are backing up.
            destinations (list of str): Paths of where you want the backup to
                be stored.
            name (str): Name of the backup, this will name the backup files.
            compression_level (int, optional): The gzip compression level that
                you want to use for the backup. Defaults to 6.
        """
        self._sources = sources
        self._destinations = destinations
        self._name = name
        self._compression_level = compression_level

    def backup(self):
        """Standard simbak backup.

        This will backup all the files defined in the sources and store them
        in a gzip'd file in each of the destinations. The name of the gzip file
        will be the name parameter suffixed with a time stamp, the format of
        the timestamp is YYYY-MM-DD--hh-mm-ss
        """
        _logger.info(f'Starting backup [{self._name}]')
        filtered_sources = _fileutil.filter_paths(self._sources)
        filtered_destinations = _fileutil.filter_paths(
            self._destinations, create=True)
        file_name = _fileutil.unique_file_name(self._name)
        _logger.info(f'Backup file name will be {self._name}')

        first_path = _fileutil.create_targz(
            sources=filtered_sources,
            destination=filtered_destinations[0],
            file_name=file_name,
            compression_level=self._compression_level
        )
        _fileutil.distribute_file(
            path=first_path,
            destinations=filtered_destinations[1:]
        )



import logging as _logging
import os as _os
import shutil as _shutil
import tarfile as _tarfile
from datetime import datetime as _datetime

_logger = _logging.getLogger(__name__)


def filter_paths(paths: list, create=False) -> list:
    """Gets only existing paths and cleans the paths to be OS compliant.

    Args:
        paths (list of str): The list of paths that needs to be filtered.
        create (bool, optional): Create a path if it doesn't exist.
            Defaults to False.

    Returns:
        list of str: The filtered list of paths.
    """
    filtered_paths = []

    for path in paths:
        normpath = _os.path.normpath(path)

        if _os.path.exists(normpath):
            filtered_paths.append(normpath)
        elif create:
            _logger.info(f'{normpath} doesn\'t exist, creating directory '
                         'with that path.')
            _os.makedirs(normpath)
            filtered_paths.append(normpath)
        else:
            _logger.warning(
                f'Failed to access {normpath}, it doesn\'t exist')

    return filtered_paths


def unique_file_name(name: str) -> str:
    """Creates a unique file name by suffixing date and time.

    Args:
        name (str): The prefix of the unique file name.

    Returns:
        str: The unique file in the format of
            <name>--YYYY-MM-DD--hh-mm-ss.tar.gz
    """
    time_suffix = _datetime.now().strftime('%Y-%m-%d--%H-%M-%S')
    file_name = f'{name}--{time_suffix}.tar.gz'
    return file_name


def distribute_file(path: str, destinations: list):
    """Distributes a backup file to other destinations.

    Args:
        path (str): Path to the file that you want to distribute.
        destinations (list of str): Paths of where you want the file to
            be stored.
    """
    file_name = _os.path.basename(path)

    for destination in destinations:
        to_path = _os.path.join(destination, file_name)
        _shutil.copyfile(path, to_path)
        _logger.info(f'Saved backup {file_name} to {destination}')


def create_targz(sources: list, destination: str, file_name: str,
                 compression_level: int = 6) -> str:
    """Creates a targz file in a single destination.

    If you try to create a targz file for a file that already exists, it
    will not overwrite that file.

    Args:
        sources (list of str): The paths of the files you want to file.
        destination (str): The path of where you want the file to be stored.
        file_name (str): The name that the file will be.
        compression_level (int): The gzip compression level that you want
            to use for the file.

    Returns:
        str: Path of the file that was created, if the file already existed,
            it will return the already existing file path.
    """
    path = _os.path.join(destination, file_name)

    try:
        tar = _tarfile.open(path, 'x:gz',
                            compresslevel=compression_level)

        for source in sources:
            basename = _os.path.basename(source)
            _logger.info(f'Compressing {source}')
            try:
                tar.add(source, basename)
            except PermissionError:
                _logger.error(f'Couldn\'t compress {source}, pemission '
                              'denied.')

        tar.close()
        _logger.info(f'Saved backup {file_name} to {destination}')
    except FileExistsError:
        _logger.error(f'Failed to create backup {file_name}, file already '
                      'exists')

    return path



import setuptools

with open('README.md', 'r') as file:
    long_description = file.read()

setuptools.setup(
    name='simbak',
    version='0.2.0',
    author='Mark Bromell',
    author_email='markbromell.business@gmail.com',
    description='A simple backup solution that\'s light and portable',
    long_description=long_description,
    long_description_content_type='text/markdown',
    python_requires='>=3.5',
    packages=setuptools.find_packages(),
    url='https://github.com/mark-bromell/simbak',
    project_urls={
        'Source Code': 'https://github.com/mark-bromell/simbak',
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    entry_points={
        'console_scripts': [
            'simbak = simbak.__main__:main',
        ],
    },
    install_requires=[
        'deprecation',
    ],
)


import os as _os
import logging as _logging
import logging.handlers as _handlers
from simbak.agent.normal import NormalAgent as _NormalAgent


__version__ = '0.2.0'

# Setting up logger
if _os.path.exists('logs/') is False:
    _os.mkdir('logs/')

_stream_formatter = _logging.Formatter('%(levelname)s: %(message)s')
rotating_file_formatter = _logging.Formatter(
    fmt='%(asctime)s - %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H-%M-%S'
)

_stream_handler = _logging.StreamHandler()
_stream_handler.setLevel(_logging.INFO)
_stream_handler.setFormatter(_stream_formatter)

_rotating_file_handler = _handlers.RotatingFileHandler(
    'logs/simbak.log', maxBytes=1000000, backupCount=20)
_rotating_file_handler.setLevel(_logging.DEBUG)
_rotating_file_handler.setFormatter(rotating_file_formatter)

_root_logger = _logging.getLogger()
_root_logger.setLevel(_logging.DEBUG)
_root_logger.addHandler(_stream_handler)
_root_logger.addHandler(_rotating_file_handler)


def backup(sources: list, destinations: list, name: str,
           compression_level: int = 6):
    """The easiest way to perform a standard backup.

    Args:
        Args:
            sources (list of str): Paths to the files that you are backing up.
            destinations (list of str): Paths of where you want the backup to
                be stored.
            name (str): Name of the backup, this will name the backup files.
            compression_level (int, optional): The gzip compression level that
                you want to use for the backup. Defaults to 6.
    """
    agent = _NormalAgent(sources, destinations, name, compression_level)
    agent.backup()



import argparse
import sys

import simbak


def parse_args(args):
    """Parses all the agruments sent through the command line.

    Returns:
        object: The parsed arguments.
    """
    parser = argparse.ArgumentParser(
        prog='simbak',
        description='Simple backup solution.',
    )
    parser.add_argument(
        '-s', '--source',
        type=str,
        nargs='+',
        help='The source(s) of the dir(s) you want to backup.',
        required=True,
    )
    parser.add_argument(
        '-d', '--destination',
        type=str,
        nargs='+',
        help='The backup destination(s) dir(s).',
        required=True,
    )
    parser.add_argument(
        '-n', '--name',
        type=str,
        help='The name of the backup.',
        required=True,
    )
    parser.add_argument(
        '-c', '--compression-level',
        type=int,
        default=6,
        help=('The compression level (1-9) of the gzip backup algorithm, '
              'default is 6.'),
    )
    return parser.parse_args(args)


def main():
    """Consider this the entry point to the command line utility."""
    args = parse_args(sys.argv[1:])
    simbak.backup(
        sources=args.source,
        destinations=args.destination,
        name=args.name,
        compression_level=args.compression_level
    )
